DOCS_BASE = (
    "This cog has docs! Check them out at\n"
    "https://vex-cogs.rtfd.io/en/latest/cogs/{}.html?utm_source=cog&utm_medium=docstring"
    "&utm_campaign=main_help"
)

CHECK = "\N{HEAVY CHECK MARK}\N{VARIATION SELECTOR-16}"
CROSS = "\N{CROSS MARK}"
